/** @(#)FlyButton.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

import java.awt.*;
import java.awt.image.PixelGrabber;
import java.awt.image.ImageObserver;
import java.awt.image.MemoryImageSource;
import java.util.*;
import brr.awt.*;

/** 
 
FlyButton is a simple subclass of FlyPanel. It can also be used as a normal
button by calling:
<pre>
setFlyButton(false);
</pre>
<p>
<applet code=brr.awt.examples.FlyButtonExample.class width=500 height=50>
<param name=fly value=true>
</applet>
<p>Source: <a href="brr/awt/examples/FlyButtonExample.java">FlyButtonExample.java</a><p>
*/

public class FlyButton extends FlyPanel {

    TextPanel bs;
    ImagePanel bi;
    Image img, grayImg;

    /**Constructor with no image.
        @param text the label to be displayed
    */
    public FlyButton(String text) {
        this(null, text);
    }
 
    /**Constructor with only an Image. Default size is 32 pixels x 32 pixels.
        @param image the image to be displayed
    */
    public FlyButton(Image image) {
        this(image, null);
    }
 
    /**Constructor with an Image and a label. 
	Default image size is 32 pixels x 32 pixels.
        @param image the image to be displayed
        @param text the label to be displayed
    */
    public FlyButton(Image image, String text) {
	this(new ImagePanel(image), new TextPanel(text));
    }

    /**Constructor with an Image and a label and flyButton specification. 
	Default image size is 32 pixels x 32 pixels.
        @param image the image to be displayed
        @param text the label to be displayed
	@param fly if true, this acts like a flybutton else as anormal Button.
    */
    public FlyButton(Image image, String text, boolean fly) {
	this(image, text);
	setFlyButton(fly);
    }

    /** For greater flexibility, use this constructor. This allows use of
	all the ImagePanel and TextPanel features.
	@param bi the preconfigured ImagePanel 
	@param bs the preconfigured TextPanel
	@see ImagePanel
	@see TextPanel
    */
    public FlyButton(ImagePanel bi, TextPanel bs) {
	add(this.bi = bi);
	add(this.bs = bs);
	img = bi.getImage();
    }

    /** For greater flexibility, use this constructor. This allows use of
	all the ImagePanel and TextPanel features.
	@param bi the preconfigured ImagePanel 
	@param bs the preconfigured TextPanel
        @param fly if true, this acts like a flybutton else as anormal Button.
	@see ImagePanel
	@see TextPanel
    */
    public FlyButton(ImagePanel bi, TextPanel bs, boolean fly) {
	this(bi, bs);
	setFlyButton(fly);
    }

    /** Draw a disabled version of the FlyButton and disable. 
    */
    public void disable() {
	super.disable();
	grayImg = createSemiTransparentImage(img);
	bi.setImage(grayImg);
	bs.disable();
	repaint();
    }

    /** Draw a normal version of the FlyButton and enable. 
    */
    public void enable() {
	super.enable();
	bi.setImage(img);
	bs.enable();
	repaint();
    }

    private Image createEmbossedImage(Image image) {
	int w = image.getWidth(this);
	int h = image.getHeight(this);
	int pixels[] = new int[w * h];
	int emb[] = new int[w * h];
	PixelGrabber pg = new PixelGrabber(image, 0, 0, w, h, pixels, 0, w) ;
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    return null;
	}
	if ((pg.status() & ImageObserver.ABORT) != 0) {
	    return null;
	}
	
	Color bg = this.getBackground();
	Color darker = bg.darker();
	Color brighter = bg.brighter();
	
	// Set the entire image to the bg color
	for (int i=0; i < w*h; ++i) emb[i] = bg.getRGB();
	
	// Pretty much a major hack here; assuming pixel[0] = the bg color!
	bg = new Color(pixels[0]);
 
	// Then fill in the outline of the image all white and
	// offset by 1 to the right and down
	for (int j = 0; j < h; j++) {
	    for (int i = 0; i < w; i++) {
		Color c = new Color(pixels[j * w + i]);
	        if (!bg.equals(c)) {
	            if (i < w-1 && j < h-1)
	                emb[(j+1) * w + i + 1] = brighter.getRGB();
	        }
	    }
	}
	
	// Then cover the original image area with a darker version of
	// the background color
	for (int j = 0; j < h; j++) {
	    for (int i = 0; i < w; i++) {
	        Color c = new Color(pixels[j * w + i]);
	            if (!bg.equals(c))
	                emb[j * w + i] = darker.getRGB();
	    }
	}
	
	return this.createImage(new MemoryImageSource(w, h, emb, 0, w));

    }

    private Image createSemiTransparentImage(Image image) {
	int w = image.getWidth(this);
	int h = image.getHeight(this);
	int pixels[] = new int[w * h];
	int emb[] = new int[w * h];
	PixelGrabber pg = new PixelGrabber(image, 0, 0, w, h, pixels, 0, w) ;
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    return null;
	}
	if ((pg.status() & ImageObserver.ABORT) != 0) {
	    return null;
	}
	
	Color bg = this.getBackground();
	Color darker = bg.darker();
	Color brighter = bg.brighter();
	
	// Set the entire image to the bg color
	for (int i=0; i < w*h; ++i) emb[i] = bg.getRGB();
	
	// Pretty much a major hack here; assuming pixel[0] = the bg color!
	bg = new Color(pixels[0]);
 
/*
	// Then fill in the outline of the image all white and
	// offset by 1 to the right and down
	for (int j = 0; j < h; j++) {
	    for (int i = 0; i < w; i++) {
		Color c = new Color(pixels[j * w + i]);
	        if (!bg.equals(c)) {
	            if (i < w-1 && j < h-1)
	                emb[(j+1) * w + i + 1] = brighter.getRGB();
	        }
	    }
	}
*/
	
	// Then cover the original image area with a tranparent version of
	// the original color
	for (int j = 0; j < h; j++) {
	    for (int i = 0; i < w; i++) {
	        Color c = new Color(pixels[j * w + i]);
	            if (!bg.equals(c))
	                emb[j * w + i] = c.getRGB() & 0x7FFFFFFF;
	    }
	}
	
	return this.createImage(new MemoryImageSource(w, h, emb, 0, w));

    }

    /** @return the font of the label
    */
    public Font getFont() {
	return bs.getFont();
    }

    /** Set the font of the label
    */
    public void setFont(Font font) {
	bs.setFont(font);
    }

    /** @return the label.
    */
    public String getText() {
	return bs.getText();
    }

    /** set the label.
    */
    public void setText(String text) {
	bs.setText(text);
    }

    /** @return the image.
    */
    public Image getImage() {
	return bi.getImage();
    }

    /** set the image.
    */
    public void setImage(Image image) {
	bi.setImage(image);
    }

};
