/** @(#)FlyPanel.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

import java.awt.*;
import java.util.*;
import brr.awt.*;

/** 
 
FlyPanel is a Panel that draws a 3D border whenever the mouse is 
inside the Panel. This can be used to create  FlyButtons.
Just add ImagePanels, TextPanels or whatever to customize your
own FlyButton. The Fly behavior can be suppressed by calling
<pre>
setFlyButton(false);
</pre>
That would cause the FlyPanel to behave like a "normal" 3D button.
 
Just use a FlyPanel as you would any other Panel.
<p>
Note: Under Java 1.0.x Components inside FlyPanel must return true for the 
mouseExit() method if there are two or more components. 
This is to prevent border flicker when mouse moves from one component to 
another. This is due to a bug/feature of the inside() method in JDK 1.0.x. <p>
The bug/feature has been debugged/removed in JDK 1.1 and above.<p>
The bug/feature does not affect FlyPanels that behave like normal buttons.<p>
<pre>
Flypanel p = new FlyPanel();
p.add(new ImagePanel(img));
</pre>
<applet code=brr.awt.examples.FlyPanelExample.class width=300 height=150>
</applet>
<p>Source: <a href="brr/awt/examples/FlyPanelExample.java">FlyPanelExample.java</a><p>

*/

public class FlyPanel extends Panel {
	
    boolean flyButton = true;
    boolean pushedIn = false;
    boolean mouseIn = false;
    int borderWidth = 2;

    /** Control FlyPanel behavior
        @param flyButton if true, this will act like a FlyButton
    */
    public void setFlyButton(boolean flyButton) {
        this.flyButton = flyButton;
    }
 
    /** @return true if this is is acting like a fly button else false
    */
    public boolean getFlyButton() { return flyButton; }
 
    public void realign() {
	Dimension d = minimumSize();
	Rectangle r = bounds();
	resize( d.width, d.height);
	//move(r.x+(r.width > d.width ? (r.width - d.width)/2 : 0), 
	//     r.y+(r.height> d.height? (r.height - d.height)/2:0));
	layout();
    }

    public boolean mouseExit(Event evt, int x, int y) {
	if (!isEnabled()) return false;
	if (!System.getProperty("java.version").startsWith("1.0") && 
		inside(x,y)) return false;
	if (mouseIn) {
	mouseIn = false;
	if(flyButton)eraseBorder(); // was drawBorder(false);
	}
	return false;
    }

    public boolean mouseEnter(Event evt, int x, int y) {
	if (!isEnabled()) return false;
	if (!mouseIn) {
	mouseIn = true;
	if(flyButton)drawBorder(true);
	}
	return false;
    }

    public boolean mouseDown(Event evt, int x, int y) {
	if (!isEnabled()) return false;
	pushedIn = true;
	drawBorder(false);
	return false;
    }

    public boolean mouseUp(Event evt, int x, int y) {
	if (!isEnabled()) return false;
	if(inside(x,y)) {
	    pushedIn = false;
	    drawBorder(true);
	    evt.target = this;
	    evt.id = Event.ACTION_EVENT;
	    postEvent(evt);
	    return true;
	} else {  
	    pushedIn = false; mouseIn = false;
	    if(!flyButton)drawBorder(true); 
	    else eraseBorder();
	}
	return false;
    }

    /** Erases the 3D border. Most applications should not call this directly.
    */
    public void eraseBorder() {
        Graphics g = this.getGraphics();
        Color oldColor = g.getColor();
	g.setColor(getBackground());
	for(int i = 0; i < borderWidth; i++) 
            g.drawRect(i, i, size().width-(2*i+1), size().height-(2*i+1));
        g.setColor(oldColor);
    }

    /** Draws the 3D border. Most applications should not call this directly.
    */
    public void drawBorder(boolean toggle) {
	Graphics g = this.getGraphics();
    	Color oldColor = g.getColor();
	g.setColor(getBackground());
	for(int i = 0; i < borderWidth; i++) 
	    g.draw3DRect(i, i, 
			size().width-(2*i+1), size().height-(2*i+1), 
			toggle);
	g.setColor(oldColor);
    }
	
    public void enable() {
        if (!isEnabled()) {
            pushedIn = false;
            super.enable();
            repaint();
         }
    }

    public void disable() {
        if (isEnabled()) {
            pushedIn = false;
            super.disable();
            repaint();
         }
    }

    public void paint(Graphics g) {
	realign();
	if(!flyButton)drawBorder(true);
	else eraseBorder();

    }
};
