/** @(#)Grid.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/
package brr.awt;

import java.awt.*;

/** 
Grid class provides a basic Grid for board games such as chess, scrabble
other games such as crossword and tic-tac-toe. etc.
<pre>
Variables that control the Grid and thier default values.

   int xsize, ysize;         // # of boxes in the crossword (no defaults)
   int squareSize = 25;      // size of each square in pixels
   int borderSize = 1;       // width of border around each square in pixels

   Color squareColor 	= Color.white;   // color of writable squares
   Color backdropColor 	= Color.black;   // color of unwritable squares
</pre>

<applet code=brr.awt.examples.GridExample.class width = 550 height = 350>
</applet>
<p>Source: <a href="brr/awt/examples/GridExample.java">GridExample.java</a><p>

*/

public class Grid extends Canvas {

   int xsize, ysize;         // # of boxes in the crossword
   int squareSize = 25;      // size of each square in pixels
   int borderSize = 1;       // width of border around each square in pixels


   Color squareColor 	= Color.white;   // color of writable squares
   Color backdropColor 	= Color.black;   // color of unwritable squares

    /** Construct a Grid with xsize columns and ysize rows.
	@param xsize number of columns
	@param ysize number of rows
    */
    public Grid(int xsize, int ysize) {
	this(xsize, ysize, 25);
    }

    /**Construct a Grid with xsize columns, ysize rows and specified squareSize.
	squareSize does not include the borderSize.
	@param xsize number of columns
	@param ysize number of rows
	@param squareSize size of the squares in pixels.
    */
    public Grid(int xsize, int ysize, int squareSize) {
	this.xsize = xsize; this.ysize = ysize; this.squareSize = squareSize;
	resize();
    }

    /** Sets the # rows and cols for this Grid.
    */
    public void setGridSize(int x, int y) {
	xsize = x; ysize = y; resize();
    }

    /** Sets squareSize.
    */
    public void setSquareSize(int squareSize) {
	this.squareSize = squareSize; resize();
    }

    /** Sets borderSize for this Grid.
    */
    public void setBorderSize(int borderSize) {
	this.borderSize = borderSize; resize();
    }

    /** Sets the background color for the Grid. The background color is the 
	color of everything except the squares.
    */
    public void setBackdropColor(Color BackdropColor) {
	this.backdropColor = backdropColor; paint(this.getGraphics());
    }

    /** Sets the square color. 
    */
    public void setSquareColor(Color squareColor) {
	this.squareColor = squareColor; paint(this.getGraphics());
    }

    /** @return borderSize
    */
    public int getBorderSize() { return borderSize;}

    /** @return squareSize
    */
    public int getSquareSize() { return squareSize;}

    /** @return backdropColor
    */
    public Color getBackdropColor() { return  backdropColor;}

    /** @return squareColor
    */
    public Color getSquareColor() { return  squareColor;}

    /** @return xsize, ysize of the Grid.
    */
    public Dimension getGridSize() { return new Dimension(xsize, ysize); }

    /** This will resize the Grid to fit the new dimensions. Called by methods 
	that modify the size variables. Not normally called by other classes.
    */
    public void resize() {
	resize(xsize*squareSize+2*borderSize,ysize*squareSize+2*borderSize);
	invalidate();
	validate();
	repaint();
    }

    /** Render a single square.
	This method essentially calls drawBackground() and drawForeground()
	Subclasses can overload this function to modify the implementation.
	@param x,y the horizontal(x) and vertical(y) location of the square to
	be painted.
    */
    public void paintSquare(int x, int y, Graphics g) {
	drawBackground(x, y, g);
	drawForeground(x, y, g);
    }

    /** Render the background of a single square.
	By default, Grid just paints the square with the background color. 
        Subclasses can overload this function to modify the implementation,
	for example, draw a background image.
        @param x,y the horizontal(x) and vertical(y) location of the square to
        be painted.
    */
    public void drawBackground(int x, int y, Graphics g) {
	Color back = squareColor;

	g.setColor(back);
	g.fillRect(x*squareSize+borderSize, y*squareSize+borderSize,
		squareSize-borderSize, squareSize-borderSize);
    }

    /** Render the foreground of a single square.
        By default, Grid does nothing.
        Subclasses can overload this function to modify the implementation,
        for example, draw a game item such as a chessman.
        @param x,y the horizontal(x) and vertical(y) location of the square to
        be painted.
    */
    public void drawForeground(int x, int y, Graphics g) {}

    /** draws a String at a Grid location. This is useful for letter Games
	such as scrabble and crossword.
        @param x,y location where the image is to be rendered. 
        x and y are horizontal and vertical square indices of the Grid.
	@param c the string will be rendered in this color.
	@param f the string will be rendered in this font.
	@param shift leave a margin of so many pixels to the left.
	@param elevation draw the string at these many pixels above the base 
		of the square.
    */
    public void drawString(String s, int x, int y, 
		    Color c, Font f, 
		    int shift, int elevation) {
	Graphics g = this.getGraphics();
        g.setFont(f);
        g.setColor(c);
        g.drawString(s, x*squareSize+borderSize+shift,
                           (y+1)*squareSize-elevation);
 
    }

    /** draws an Image at a Grid location resized to fit into the square. 
	This is useful for games such as chess.
        @param image the image that will be rendered 
	@param x,y location where the image is to be rendered. 
	x and y are horizontal and vertical square indices of the Grid.
    */
    void drawImage(Image image, int x, int y) { 
	Graphics g = this.getGraphics();
        g.drawImage(image,
                    x*squareSize+borderSize, y*squareSize+borderSize,
                    squareSize-2*borderSize, squareSize-2*borderSize,
                    this);
    }

    public void paint(Graphics g) {
	if ( g == null) return;
       	g.setColor(backdropColor);
	g.fillRect(0, 0, squareSize*xsize+borderSize,
			   squareSize*ysize+borderSize); // was 2*borderSize
	for(int y = 0; y < ysize; y++)
	   for( int x = 0; x < xsize; x++) 
	       paintSquare(x, y, g);

    }

};
