/**
@(#) ImageLoader.java		May 11 1997	
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/
package brr.awt;

import brr.awt.*;
import java.awt.*;
import java.applet.*;
import java.net.*;
import java.io.*;


/**
Loads Images from across the network. Displays a progress bar to show status.
This Frame is visible by default. It may be hidden by calling setVisible(false);
The ImageLoader can be used repeatedly until all images are loaded. It may
be destroyed by calling it's dispose() method.
<p>
<applet code=brr.awt.examples.ImageLoaderExample.class height = 150 width = 150>
</applet>
<p>Source: <a href="brr/awt/examples/ImageLoaderExample.java">ImageLoaderExample.java</a><p>

*/

public class ImageLoader extends java.awt.Frame {
    String codeBase;
    int loaded = 0;
    int toLoad = 0;
    int linesToLoad = 0;
    int linesLoaded = 0;
    ProgressBar p;
    TextPanel bs;

    /** Construct an ImageLoader for this codebase with the default message.
	Default message is "Loading Images..."<p>
	@param codeBase a String that can be converted to a URL.
	The images to be loaded are specified relative to this location.<p>
	Example:<p>
	<pre>
	ImageLoader il = new ImageLoader("http://www.javasoft.com/images/");
	</pre>
    */
    public ImageLoader(String codeBase){ 
	this(codeBase,  new TextPanel("Loading Images...",
			new Font("Helvetica", Font.BOLD, 18)));
    }

    /** Construct an ImageLoader for this codebase with the specified message
	@param headLine is the new TextPanel
        @param codeBase a String that can be converted to a URL.
        The images to be loaded are specified relative to this location.<p>
        Example:<p>
        <pre>
        ImageLoader il = new ImageLoader("http://www.javasoft.com/images/", 
			"Loading pictures. Please Wait...");
        </pre>
    */
    public ImageLoader(String codeBase, TextPanel headLine) { 
	this.codeBase = codeBase; 
	setTitle("ImageLoader");
        setLayout(new BorderLayout());
        add("North", bs = headLine);
        add("South", p = new ProgressBar(200, 30));
        pack();
    }

    /** Sets the label on top of the ProgressBar.
	@param text The new label
    */
    public void setText(String text) { bs.setText(text); }

    /**Load images specified in a String array into a section of an Image array.
        @param images the image array
        @param imageNames the image names for corresponding images in the image
                array.
        <p>Example:<p>
        <pre>
        ImageLoader il = new ImageLoader("http://www.javasoft.com/images/");
        String images[] = { "open.gif", "close.gif", "new.gif", "save.gif" };
        Image iconImages = new Images[4];
        il.loadImages(images, iconImages);
        </pre><p>
        The code above would load all 4 images into the array.
    */
    public void loadImages(Image[] images,
                String[] imageNames) {
        this.loadImages(images, 0, images.length-1, imageNames);
    }

    /**Load images specified in a String array into a section of an Image array.
        @param images the image array
        @param imageNames the image names for corresponding images in the image
		array.
        @param beginIndex/endIndex the images are loaded for these indices only.
        <p>Example:<p>
        <pre>
        ImageLoader il = new ImageLoader("http://www.javasoft.com/images/");
	String images[] = { "open.gif", "close.gif", "new.gif", "save.gif" };
        Image iconImages = new Images[4];
        il.loadImages(iconImages, 2, 3, iconImages);
        </pre><p>
        The code above would load close.gif and new.gif into the array.
    */
    public void loadImages(Image[] images,
		int beginIndex, int endIndex,
                String[] imageNames) {
        loaded = 0; toLoad = endIndex - beginIndex+1;
        p.setPercent(0);
        MediaTracker mt = new MediaTracker(this);
        for(int i = beginIndex; i <= endIndex; i++) {
            URL imgURL = null ;
            try {
                imgURL = new URL(codeBase+imageNames[i]);
            } catch (MalformedURLException me) { me.printStackTrace(); }
            images[i] = Toolkit.getDefaultToolkit().getImage(imgURL);
            prepareImage(images[i], this);
            mt.addImage(images[i], 0);
        }
        try { mt.waitForAll(); } catch (InterruptedException ie) { }
    }

    /** Load images into an array.
	@param images the image array
	@param prefix/suffix the image name is constructed by inserting the
	array index in between the prefix and the suffix.
	<p>Example:<p>
	<pre>
	ImageLoader il = new ImageLoader("http://www.javasoft.com/images/");
	Image iconImages = new Images[20];
	il.loadImages(iconImages, "icon", ".gif"); // dont forget the "." in suffix
	</pre><p>
	The code above would load icon0.gif, icon1.gif .... icon19.gif
	into the array.
    */
    public void loadImages(Image[] images, 
		String prefix, String suffix) {
	this.loadImages(images, 0, images.length-1, prefix, suffix);
    }

    /** Load images into a section of an array.
        @param images the image array
        @param prefix/suffix the image name is constructed by inserting the
        array index in between the prefix and the suffix.
	@param beginIndex/endIndex the images are loaded for these indices only
        <p>Example:<p>
        <pre>
        ImageLoader il = new ImageLoader("http://www.javasoft.com/images/");
        Image iconImages = new Images[20];
        il.loadImages(iconImages, 3, 10, "icon", ".gif");
        </pre><p>
        The code above would load icon3.gif, icon4.gif .... icon10.gif
        into the array.
    */
    public void loadImages(Image[] images, 
		int beginIndex, int endIndex, 
		String prefix, String suffix) {
    	loaded = 0; toLoad = endIndex - beginIndex+1;
	p.setPercent(0);
	MediaTracker mt = new MediaTracker(this);
	for(int i = beginIndex; i <= endIndex; i++) {
	    URL imgURL = null ;
	    try {
		imgURL = new URL(codeBase+prefix+i+suffix);
	    } catch (MalformedURLException me) { me.printStackTrace(); }
	    images[i] = Toolkit.getDefaultToolkit().getImage(imgURL);
	    prepareImage(images[i], this);
	    mt.addImage(images[i], 0);
	}
	try { mt.waitForAll(); } catch (InterruptedException ie) { }
    }
  
    /** Load a single image.
	@param name the image to be loaded relative to the codeBase
	@return the image at the URL.
    */
    public Image loadImage(String name) {
	Image image;
    	loaded = 0; 
	p.setPercent(0);
        MediaTracker mt = new MediaTracker(this);
        URL imgURL = null ;
        try {
            imgURL = new URL(codeBase+name);
        } catch (MalformedURLException me) { me.printStackTrace(); }
        image = Toolkit.getDefaultToolkit().getImage(imgURL);
	toLoad = image.getHeight(this);
        this.prepareImage(image, this);
        mt.addImage(image, 0);
        try { mt.waitForAll(); } catch (InterruptedException ie) { }
	return image;
    }

    public boolean imageUpdate(Image  img, int  infoflags,
            int x, int y, int  width, int  height) {
	if((infoflags & HEIGHT) == HEIGHT) 
		return false;
/*
	if((infoflags & SOMEBITS) == SOMEBITS)
		p.setPercent(loaded*100/toLoad+ (y*100)/(height*toLoad));
*/
	if((infoflags & ALLBITS) == ALLBITS && toLoad != 0) 
		p.setPercent(++loaded*100/toLoad);
	return true;
    }
/*
        if ((infoflags & ABORT) != 0)
                return true;
        if ((infoflags & ABORT & ERROR & FRAMEBITS & ALLBITS) == 0) {
            if ((infoflags & WIDTH) != 0) imageWidth = width;
            if ((infoflags & HEIGHT) != 0) imageWidth = height;
        }
        if ((infoflags & ALLBITS) != 0) {
//          this.getGraphics().drawImage((Image)image, x, y, this);
                return false;
        } else {
                return true;
        }
    }
*/

};
