/** @(#)ImagePanel.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

import java.awt.*;
import java.util.*;
import brr.awt.*;

/** 
 
ImagePanel is a class that allows treating an Image as a Component.
This allows us to add an Image to most Applets and Frames without
the bother of deriving a class or writing paint() methods.<p>
 
Just add a ImagePanel as you would any other Component.
<pre>
add(new ImagePanel(img));
</pre>
<p>A multi-frame image such as an animated GIF will result in the 
	animation being played.
<applet code=brr.awt.examples.FlyButtonExample.class width = 200 height=200>
<param name=fly value=false>
</applet>
<p>Source: <a href="brr/awt/examples/FlyButtonExample.java">FlyButtonExample.java</a><p>
*/

public class ImagePanel extends java.awt.Canvas {

    boolean fit = true;
    static final int DEFAULT_IMAGE_HEIGHT = 32;
    static final int DEFAULT_IMAGE_WIDTH = 32;
    int imageHeight;
    int imageWidth;
    Image image = null;

    /** Specify whether to resize the image to fit into the imageHeight 
	and imageWidth
	@param fit if fit is true, the Image will be resized to make it fit 
	into the height and width of the ImagePanel. 
	If not, it will be rendered at it's
    	real size. fit = true is used for displaying many images at the same
	size as in picture index, toolbars etc.
    */
    public void setFit(boolean fit) { this.fit = fit; }

    /** Void Constructor for use in Arrays. The Image and size can be set later.
    */
    public ImagePanel() {
	this(null, DEFAULT_IMAGE_WIDTH, DEFAULT_IMAGE_HEIGHT, true);
    }

    /**Constructor with the default size. Default size is 32 pixels x 32 pixels.
	@param image the image to be displayed
    */
    public ImagePanel(Image image) {
	this(image, DEFAULT_IMAGE_WIDTH, DEFAULT_IMAGE_HEIGHT, true);
    }

    /** Constructor with the default size and specified fit.
	@param image the image to be displayed.
	@param fit specifies whether to resize the image to fit the Banner.
    */
    public ImagePanel(Image image, boolean fit) {
	this(image, DEFAULT_IMAGE_WIDTH, DEFAULT_IMAGE_HEIGHT, fit);
    }

    /** Constructor with Image and height.  
	@param image the image to be displayed
	@param width width of the banner
	@param height height of the banner
        <p> Width and height represent the area reserved for the image
	to be displayed. The image is rendered at it's original size unless
	fit is true.
	@see setFit
    */
    public ImagePanel(Image image, int width, int height) {
	this(image, width, height, true);
    }

    /** The essential complete constructor.  
        @param image the image to be displayed
        @param width width of the banner
        @param height height of the banner
	@param fit if true, image is resized to fit the Banner
        <p> Width and height represent the area reserved for the image
        to be displayed. The image is rendered at it's original size unless
        fit is true.
    */
    public ImagePanel(Image image, int width, int height, boolean fit) {
        this.fit = fit;
        if (fit) { imageWidth  = width; imageHeight = height; }
	setImage(image);
    }

    /** Set the Image to be rendered. A multi-frame image such as an animated 
	GIF will result in the animation being played.
    */
    public void setImage(Image image) { 
        this.image = image;
	if (image != null) {
            prepareImage(image, this);
	    if (!fit) { 
		imageWidth  = image.getWidth(this); 
		imageHeight = image.getHeight(this); 
	    }
	}
	setImageSize(imageWidth, imageHeight);
    }

    public boolean imageUpdate(Image img, int infoflags, int x, int y,
		int width, int height) {
	if ( fit) { super.imageUpdate(img, infoflags, x, y, width, height);}
	if ((infoflags & (WIDTH | HEIGHT) ) == (WIDTH|HEIGHT) ) {
	    setImageSize(width, height);
	    super.imageUpdate(img, infoflags, x, y, width, height);
	    return false;
	} else {
	    super.imageUpdate(img, infoflags, x, y, width, height);
	    return true;
	}
    }

    /** The different setImageSize methods are used to resize the ImagePanel 
	@param imageWidth the new width of the image
	@param imageHeight the new height of the image
    */
    public void setImageSize(int imageWidth, int imageHeight) { 
	this.imageWidth = imageWidth;
	this.imageHeight = imageHeight;
	resize(imageWidth, imageHeight);
	//invalidate();
	//validate();
	repaint();
    }

    /** @param d Dimension representing the new size of the Banner*/
    public void setImageSize(Dimension d) { 
	this.setImageSize(d.width, d.height);
    }

    /** @return the width and height of the ImagePanel
	@see java.awt.Dimension
    */
    public Dimension getImageSize() { 
	return new Dimension(imageWidth, imageHeight);
    }

    /** @return the Image displayed */
    public Image getImage() { return image; }

    /** Sets the width of the ImagePanel
	@param imageWidth width of the Image to be displayed. The ImagePanel
	resizes to the new width
    */
    public void setImageWidth(int imageWidth) { 
	this.setImageSize(imageWidth, imageHeight);
    }

    /** Sets the height of the ImagePanel
	@param imageHeight Height of the Image to be displayed. The ImagePanel
	resizes to the new height
    */
    public void setImageHeight(int imageHeight) { 
	this.setImageSize(imageWidth, imageHeight);
    }

    /** @return the width of the ImagePanel
    */
    public int getImageWidth() { return imageWidth;}

    /** @return the height of the ImagePanel
    */
    public int getImageHeight() { return imageHeight;}

    /** Needed so FlyPanel does not erase border
    */
    public boolean mouseExit(Event e, int x, int y) {
	return true;
    }

    public void paint(Graphics g) {
	if (image != null ) {
	   if (fit)g.drawImage(image, 0, 0, imageWidth, imageHeight, this);
	   else    g.drawImage(image, 0, 0, this);
	//g.drawImage(image, 0, 0, this);
	}
    }
	
};
