/** @(#)TextPanel.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

import java.awt.*;
import brr.awt.*;

/**
Similar to java.awt.Label except that you have control over width (in pixels)
that it occupies. This class was written to serve as head lines in Frames and 
status messages which need to change while not doing a reLayout.
<p>
<applet code=brr.awt.examples.TextPanelExample.class height=100 width=200>
</applet>
<p>Source: <a href="brr/awt/examples/TextPanelExample.java">TextPanelExample.java</a><p>

*/

public class TextPanel extends java.awt.Canvas {
    public static final Font DEFAULT_FONT = 
		new Font("Helvetica", Font.BOLD, 12);
    Font font = DEFAULT_FONT;
    FontMetrics fm;
    int minWidth = 0;
    String text = "";
    int align = Align.CENTER;

    /** Constructs a banner string with "" as the label. This constructor is for
	 arrays etc 
    */
    public TextPanel() {
	this("", DEFAULT_FONT);
    }

    /** Constructs a banner string with a label. 
	@param text the text of the label
    */
    public TextPanel(String text) {
	this(text, DEFAULT_FONT);
    }

    /** Constructs a banner string with specified label and Font. 
        @param text the text of the label
	@param font the Font to draw the label with
    */
    public TextPanel(String text, Font font) {
	super.setFont(this.font = font);
    	this.text = text == null? "" : text; 
	recalcMetrics();
    }

    /** Changes the label of the TextPanel 
        @param text the new label
    */
    public void setText(String text) {
    	this.text = text; 
	recalcMetrics();
	repaint();
    }

    /** Changes the Font of the TextPanel. Will resize if necessary.
        @param font the Font to draw the label with.
    */
    public void setFont(Font font) {
	super.setFont(this.font = font);
	recalcMetrics();
    }

    /** Recalculates the width and the height necessary to draw the label.
	This method is called by setText and setFont automatically.
	This is not normally called by users.
    */
    public void recalcMetrics() {
        fm = this.getFontMetrics(font);
	int textWidth = fm.stringWidth(text);
	resize(textWidth  > minWidth ? textWidth : minWidth, fm.getHeight());
    }

    /** Sets the Alignment of the TextPanel. 
	@param align the alignment constant defined in brr.awt.Align
	@see Align
    */
    public void setAlign(int align) { 
	this.align = align;  
	recalcMetrics(); 
	repaint();
    }

    /** Sets the minimum width of the TextPanel. The TextPanel will 
	not size to less than this width. This avoids a reLayout
	if the label is changed to a shorter string 
	@param minWidth the new minimum width of the Banner
    */
    public void setMinWidth(int minWidth) { 
	this.minWidth = minWidth; 
	recalcMetrics(); 
	repaint();
    }

    /** @return the minimum width. The minimum width is the # pixels that
	the TextPanel will have even when the label requires less.
    */
    public int getMinWidth() { return minWidth; }

    /** @return the current label 
    */
    public String getText() { return text; }

    /** @return the current alignment 
	@see Align
    */
    public int getAlign() { return align; }

    /** gives an embossed look to the label 
    */
    public void disable() {
	super.disable();
	repaint();
    }

    /** label is set to normal
    */
    public void enable() {
	super.enable();
	repaint();
    }

    /** label is enabled/disabled depending on parameter 
	@param enable if true, enabled else disabled
    */
    public void enable(boolean enable) {
        super.enable(enable);
        repaint();
    }

    /** Needed so FlyPanel does not erase border
    */
    public boolean mouseExit(Event e, int x, int y) {
        return true;
    }

    public void paint(Graphics g) {
        if (text != null) {
	    int x = Align.adjustAlign(align, fm.stringWidth(text), 
					this.size().width );
	    Color old = g.getColor();
	    //g.setColor(getForeground());
	    if (!isEnabled()) {
		g.setColor(Color.white);
		g.drawString(text, 1+x, 1+fm.getHeight() - fm.getDescent());
		g.setColor(getBackground().darker());
	    }
	    g.drawString(text, x, fm.getHeight() - fm.getDescent());
	    g.setColor(old);
	}
    }
};

		


