/** @(#) Protocol.java	1997 Jul 29 
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.net;

import brr.util.Debug;
import java.lang.Thread;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;

/**
    Protocol is used to write Servers such as Web Servers, SQL servers
    and also implement collaboratory applications. This class takes care of
    sending the right message to the right Object. This means <strong>you need
    to only implement the message action </strong> and leave the interpreting
    of the message to the Protocol class.<p>

    Check out a simple example program
    <a href="brr/net/examples/ProtocolExample.java">ProtocolExample.java</a><P>

    Check out how to use this class to build a simple Web Server:
    <a href="brr/net/examples/GetServer.java">GetServer.java</a><br>
    (Really simple! Only GET, no cgi, no servlets etc.)<p>

    Protocol is a filter that works between an InputStream and an OutputStream, 
    and these can be
    changed programatically, by calling the setInputStream() and 
    setOutputStream() methods. This makes the Protocol class very flexible.
    You can easily insert other filters such as a Protocol debugger or 
	encryption/security module
    or even just a logging facility between the streams.<p>
    For example:
<pre>
    Base situation:

    .        Request
    client <----------> Protocol 
    .        response

    Security enhanced:

     .       Encrypted Req.                     Decrypted Req.
     client <---------------> Security Manager <--------------> Protocol
     .       Encrypted Resp.                        Response
</pre>
*/
public class Protocol extends Thread {
    Hashtable protocolTable = new Hashtable();
    ProtocolHandler defaultProtocolHandler = null;
    Debug d = new Debug();

    InputStream is;
    OutputStream os;
    DataInputStream dis;
    DataOutputStream dos;

    String request;
    String requestType;

    boolean caseSensitive = false;
    boolean keepAlive = true;
    long timeOut = 2000;
 
    long markTime;
    ProtocolHandler newph;

    /** Constructs a protocol Dispatcher. 
	@param is Client requests are read from this InputStream.
	@param os ProtocolHandler output is sent to this OutputStream.
	@see ProtocolHandler
    */
    public Protocol(InputStream is, OutputStream os) {
	setInputStream(is);
	setOutputStream(os);
    }

    /** Sets the timeOut property. The Protocol is terminated after 
    <em>timeOut</em> milliseconds of inactivity.
    */
    public void setTimeOut(long timeOut) { this.timeOut = timeOut; }

    /** @return the timeOut property.
    */
    public long getTimeOut() { return timeOut; }

    /** Sets the keepAlive property. If the keepAlive property is true, then
	multiple requests are accepted. The Protocol terminates after inactivity
	specified by the setTimeOut property. If keepAlive is false, 
	the Protocol terminates after the first request.
    */
    public void setKeepAlive(boolean keepAlive) { this.keepAlive = keepAlive; }

    /** @return the value of the keepAlive property.  If the keepAlive property 
	is true, then
	multiple requests are accepted. The Protocol terminates after inactivity
	specified by the setTimeOut property. If keepAlive is false, 
	the Protocol terminates after the first request.
    */
    public boolean getKeepAlive() { return keepAlive; }

    /** Sets the InputStream to read from. Service requests are read from 
	this stream.
    */
    public void setInputStream(InputStream is) { 
	this.is = is; 
	dis = new DataInputStream(is);
	for( Enumeration e = protocolTable.elements(); e.hasMoreElements(); ) {
	    ProtocolHandler p = (ProtocolHandler)e.nextElement();
	    p.setInputStream(getInputStream());
	}
    }
 
    /** Sets the case sensitivity of the Request type. If true, <b>get, Get and
	GET</b> are all different, else they're all the same.
    */
    public void setCaseSensitive(boolean caseSensitive) { 
	this.caseSensitive = caseSensitive; 
    }
 
    /** @return the case sensitivity of the Request type.
     If true, <b>get, Get and GET</b> are all different, 
     else they're all the same.
    */
    public boolean getCaseSensitive() { return caseSensitive; }
 
    /** Sets the OutputStream to write to. Responses to service requests 
	are written to this stream.
    */
    public void setOutputStream(OutputStream os) { 
	this.os = os; 
	dos = new DataOutputStream(os);
	for( Enumeration e = protocolTable.elements(); e.hasMoreElements(); ) {
	    ProtocolHandler p = (ProtocolHandler)e.nextElement();
	    p.setOutputStream(getOutputStream());
	}
    }
 
    /** @return the InputStream to read from. Service requests are read from 
        this stream.
    */
    public InputStream getInputStream() { return is; }
 
    /** @return the OutputStream to write to. Responses to service requests 
        are written to this stream.
    */
    public OutputStream getOutputStream() { return os; }
 
    /** @return the default ProtocolHandler for Client Requests that have
	no assigned handler.
    */
    public ProtocolHandler getDefaultProtocolHandler() {
        return defaultProtocolHandler;
    }
 
    /** Sets the default ProtocolHandler for Client Requests that have
	no assigned handler.
    */
    public void setDefaultProtocolHandler(ProtocolHandler protoHandler) {
        defaultProtocolHandler = protoHandler;
        if (protoHandler.getInputStream() == null) 
            protoHandler.setInputStream(getInputStream());
        if (protoHandler.getOutputStream() == null)
            protoHandler.setOutputStream(getOutputStream());
    }

    /** @return The ProtocolHandler for a given request.
    */
    public ProtocolHandler getProtocolHandler(String protocolName) {
	return (ProtocolHandler)protocolTable.get(protocolName);
    }

    /** Sets The ProtocolHandler for a given request. Use this to dynamically
	change your protocol.
    */
    public void setProtocolHandler(ProtocolHandler protoHandler,
						String protocolName) {
	protocolTable.put(protocolName, protoHandler);
        if (protoHandler.getInputStream() == null) 
            protoHandler.setInputStream(getInputStream());
        if (protoHandler.getOutputStream() == null)
            protoHandler.setOutputStream(getOutputStream());
    }

    /** Sets The ProtocolHandlers for a given set of requests. Use this to 
	dynamically change your protocol.
    */
    public void setProtocolHandler(ProtocolHandler protoHandler,
						String[] protocolNames) {
	for( int i = 0; i < protocolNames.length; i++) {
	    setProtocolHandler(protoHandler, protocolNames[i]);
	}
    }

    /** @return The ProtocolHandlers for a given set of requests.
    */
    public ProtocolHandler[] getProtocolHandler(String[] protocolNames) {
	ProtocolHandler ph[] = new ProtocolHandler[protocolNames.length];
	for( int i = 0; i < protocolNames.length; i++) {
	    ph[i] = (ProtocolHandler)protocolTable.get(protocolNames[i]);
	}
	return ph;
    }

    /** Removes the ProtocolHandler for a given set of requests.
    */
    public void removeProtocolHandler(String[] protocolNames) {
	for( int i = 0; i < protocolNames.length; i++) {
	    protocolTable.remove(protocolNames[i]);
	}
    }

    /** Removes the ProtocolHandler for a given request.
    */
    public void removeProtocolHandler(String protocolName) {
	protocolTable.remove(protocolName);
    }

    public void run() {
	try {
	do {
	    if (dis.available() > 0) {
		request = dis.readLine();
		//d.println(request);
		if (request != null) {
		  try {
		    StringTokenizer st = new StringTokenizer(request);
		    requestType = st.nextToken();
		    if (getCaseSensitive()) 
			 requestType = requestType.toUpperCase();
		    ProtocolHandler ph = 
			(ProtocolHandler)protocolTable.get(requestType);
		    if ( ph != null) {
			handleProtocol(ph);
		    } else if (defaultProtocolHandler != null) {
			handleProtocol(defaultProtocolHandler);
		    }
		  } catch(NoSuchElementException e) { }
		}
	    } else {
		try {  sleep(500); } catch (Exception e) { }
	    }
	} while(newph == null || newph.isAlive() || 
		getKeepAlive() && 
		( timeOut < 0 || 
		    (System.currentTimeMillis() - timeOut) < markTime
		)
		 );
		// last condition is tested only after the first two...
	newph.getInputStream().close();
	} catch (IOException iox) { iox.printStackTrace(); stop(); }
	
    }

    /** The actual work of dispatching is done here. Subclasses may override 
	this to provide a different implementation and such.
    */
    public void handleProtocol(ProtocolHandler ph) {
	try {
	newph = (ProtocolHandler) ph.clone();
	// In Protocols subclassed from this, some other streams can be 
	// assigned here. This allows for creating protocol monitors etc.
	newph.setRequest(request);
	newph.setRequestType(requestType);
	newph.setProtocol(this);
	newph.start();
	while(newph.isAlive()) { try{ sleep(100); } catch(Exception e) {}}
	markTime = System.currentTimeMillis();
	} catch (CloneNotSupportedException e) {e.printStackTrace();}

    }
    
};
